==> lm_sensors-1.3.6.tgz: lm_sensors-1.3.6/lm75.c <==
/* -*- linux-c -*- */


/*
    lm75.c - Part of a Linux module for reading sensor data.
    Copyright (c) 1998  Philip Edelbrock <phil@netroedge.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/*
 *  LM75 sensor driver for LINUX
 *
 * Copyright (c) 1998 Philip Edelbrock
 * 
 * part of a module which provides a 
 * /proc/sensors file which shows
 * some information about your system.
 * Voltages, temperature and fan speeds
 * are currently reported.
 *
 *
 * please report any bugs/patches to me:
 *  phil@netroedge.com
 *
 *
 * Reference:
 *   National Semiconductor
 *   LM75 Digital Temperature Sensor and Thermal Watchdog with
 *   Two-Wire Interface, PDF, April 1997.
 *   http://www.national.com/ds/LM/LM75.pdf
 *
 *   Intel
 *   82371AB PCI-TO-ISA / IDE XCELERATOR (PIIX4), PDF, April 1997.
 *   ftp://ftp.intel.com
 *   
 *   System Management Bus Specification, Rev. 1.0, 1996.
 *   Contributors:
 *   Benchmarq Microelectronics Inc ., Duracell Inc.,
 *   Energizer Power Systems, Intel Corporation, Linear Technology Corporation,
 *   Maxim Integrated Products, Mitsubishi Electric Corporation,
 *   National Semiconductor Corporation, Toshiba Battery Co.,
 *   Varta Batterie AG, All rights reserved.
 *   http://www.sbs-forum.org/
 */

#include <linux/config.h>

/* The next define disables the redefinition of kernel_version. As it is
   already defined and initialized in lm78.c, this would result in twice
   defined symbols. We must include <linux/version.h> by hand now. */
#define __NO_VERSION__

/* Hack to allow the use of CONFIG_MODVERSIONS. In the kernel itself, this
   is arranged from the main Makefile. */
#if CONFIG_MODVERSIONS==1
#define MODVERSIONS
#include <linux/modversions.h>
#endif

#include <linux/module.h>
#include <linux/proc_fs.h>
#include <linux/ioport.h>
#include <asm/io.h>
#include <linux/pci.h>
#include <linux/delay.h>
#include "main.h"
#include "lm75.h"
#include "smbus.h"
#include "lmconfig.h"

/* Functions used only internally */
static int calc_temp(int value);


/* LM75 Initialization */
int LM75_Init () {

  int i;

#ifndef SMBUS_MODULE
  if (! SMBus_Initialized)
    return -ENODEV;
#endif

/* Set the limits on LM75's and determine which once are available. */
  for (i=0x48;i <= 0x4F; i++) { 
    if (SMBus_Read(i,0,WORD_DATA) == -1) { /* Not available */ 
      LM_Sensor_Present.lm75 &= ~ (1 << (i - 0x48));
      LM_Sensor_Config.selected.lm75 &= ~ (1 << (i - 0x48));
    }
    else { /* Test for existance first */
      SMBus_Write(i,1 | (00 << 8),WORD_DATA); /* Set Config reg to 0x00 */
      LM75_Set(LM_PROBE_LM75_OS,i-0x48,LM75_TEMP_INIT_OS);
      LM75_Set(LM_PROBE_LM75_HYST,i-0x48,LM75_TEMP_INIT_HYST);
    }
  }
  return 0;
}

void LM75_Cleanup () {
#ifndef SMBUS_MODULE
   SMBus_Cleanup();
#endif
}

void LM75_Set(int what, int nr, long value) {

      if ( (nr < 0) || (nr > 7))  /* nr out of range */
        return;
      value = value / 5 /* Now in half-degrees */;
      if (value < 0)
        value+=0x200; 
      value = ((value & 0x1fe)>>1)+ ((value & 1)<<15); /* Now in SMBus format */
      if (what == LM_PROBE_LM75_HYST) { /* Set Thyst=value */
        SMBus_Write(nr + 0x48,2 | (value << 8), WORD_DATA); 
        LM_Sensor_Data.lm75[nr].temphyst = value;
      }
      else if (what == LM_PROBE_LM75_OS) { /* Set Tos=value */
        SMBus_Write(nr + 0x48,3 | (value << 8), WORD_DATA); 
        LM_Sensor_Data.lm75[nr].tempos = value;
      }
}

long LM75_Get(int what, int nr) {
    if ( (nr < 0) || (nr > 7))  /* nr out of range */
      return 0;
    if (what == LM_PROBE_LM75_VALUE)
      return calc_temp(LM_Sensor_Data.lm75[nr].temp);
    else if (what == LM_PROBE_LM75_HYST)
      return calc_temp(LM_Sensor_Data.lm75[nr].temphyst);
    else if (what == LM_PROBE_LM75_OS)
      return calc_temp(LM_Sensor_Data.lm75[nr].tempos);
    else
      return 0; /* what out of range */
  
}

/* Input: a LM75 register value.
   Output: the temperature in tenth of degrees */
int calc_temp(int value) {
   value=value >> 7;
   if (value > 0x0191)
      value= - ( 0x0200 - value );
   return value*5;
}

int LM75_Print_Values (char *buf) {
  int i,value,value2,value3;
  int len=0;

  if (LM_Sensor_Data.valid) {

    for (i=0; i < MAX_LM75 ; i++) 
      if (LM_Sensor_Config.selected.lm75 & (1 << i)) {
        value=LM75_Get(LM_PROBE_LM75_VALUE,i);
        value2=LM75_Get(LM_PROBE_LM75_OS,i);
        value3=LM75_Get(LM_PROBE_LM75_HYST,i);
        len += sprintf(buf+len, 
                      "%s reports %d.%d degrees C (Tos=%d.%d, Thyst=%d.%d)\n",
                      LM_Sensor_Config.lm75_label[i],
                      value / 10, abs(value) % 10,
                      value2 / 10, abs(value2) % 10, 
                      value3 / 10, abs(value3) % 10);
#ifdef DEBUG
        len += sprintf(buf+len, "%s reg 1 (configuration register) reports 0x%X\n",
               LM_Sensor_Config.lm75_label[i],LM_Sensor_Data.lm75[i].config);
#endif
      }
  }
  return len;
}

void LM75_Update_Values (void) {

  int i;
  int value,smbus_addr;

/* Some very ugly code: if we have probed a LM75 location before, and 
   nothing was found, we do not probe it again. */
  for (i = 0 ; i < 8 ; i ++)
    if (LM_Sensor_Present.lm75 & (1 << i)) {
      smbus_addr=i+0x048; 
      /* Clear configuration reg */
      SMBus_Write(smbus_addr,0x000001,BYTE_DATA);
      value=SMBus_Read(smbus_addr,0,WORD_DATA);
      if (value != -1) {
        LM_Sensor_Data.lm75[i].temp=value;
#ifdef DEBUG
        LM_Sensor_Data.lm75[i].config=SMBus_Read(smbus_addr,1,WORD_DATA);
#endif
        LM_Sensor_Data.lm75[i].temphyst=SMBus_Read(smbus_addr,2,WORD_DATA);
        LM_Sensor_Data.lm75[i].tempos=SMBus_Read(smbus_addr,3,WORD_DATA);
      } else {
        LM_Sensor_Present.lm75 &= ~ (1 << i);
        LM_Sensor_Config.selected.lm75 &= ~ (1 << i);
      }
    }
}

==> lm_sensors-1.3.7.tgz: lm_sensors-1.3.7/lm75.c <==
/* -*- linux-c -*- */


/*
    lm75.c - Part of a Linux module for reading sensor data.
    Copyright (c) 1998  Philip Edelbrock <phil@netroedge.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/*
 *  LM75 sensor driver for LINUX
 *
 * Copyright (c) 1998 Philip Edelbrock
 * 
 * part of a module which provides a 
 * /proc/sensors file which shows
 * some information about your system.
 * Voltages, temperature and fan speeds
 * are currently reported.
 *
 *
 * please report any bugs/patches to me:
 *  phil@netroedge.com
 *
 *
 * Reference:
 *   National Semiconductor
 *   LM75 Digital Temperature Sensor and Thermal Watchdog with
 *   Two-Wire Interface, PDF, April 1997.
 *   http://www.national.com/ds/LM/LM75.pdf
 *
 *   Intel
 *   82371AB PCI-TO-ISA / IDE XCELERATOR (PIIX4), PDF, April 1997.
 *   ftp://ftp.intel.com
 *   
 *   System Management Bus Specification, Rev. 1.0, 1996.
 *   Contributors:
 *   Benchmarq Microelectronics Inc ., Duracell Inc.,
 *   Energizer Power Systems, Intel Corporation, Linear Technology Corporation,
 *   Maxim Integrated Products, Mitsubishi Electric Corporation,
 *   National Semiconductor Corporation, Toshiba Battery Co.,
 *   Varta Batterie AG, All rights reserved.
 *   http://www.sbs-forum.org/
 */

#include <linux/config.h>

/* The next define disables the redefinition of kernel_version. As it is
   already defined and initialized in lm78.c, this would result in twice
   defined symbols. We must include <linux/version.h> by hand now. */
#define __NO_VERSION__

/* Hack to allow the use of CONFIG_MODVERSIONS. In the kernel itself, this
   is arranged from the main Makefile. */
#if CONFIG_MODVERSIONS==1
#define MODVERSIONS
#include <linux/modversions.h>
#endif

#include <linux/module.h>
#include <linux/proc_fs.h>
#include <linux/ioport.h>
#include <asm/io.h>
#include <linux/pci.h>
#include <linux/delay.h>
#include "main.h"
#include "lm75.h"
#include "smbus.h"
#include "lmconfig.h"

/* Functions used only internally */
static int calc_temp(int value);

#ifndef SMBUS_MODULE
extern int SMBus_Initialized;
#endif

/* LM75 Initialization */
int LM75_Init () {

  int i;

#ifndef SMBUS_MODULE
  if (! SMBus_Initialized)
    return -ENODEV;
#endif

/* Set the limits on LM75's and determine which once are available. */
  for (i=0x48;i <= 0x4F; i++) { 
    if (SMBus_Read(i,0,SMBUS_WORD_DATA) == -1) { /* Not available */ 
      LM_Sensor_Present.lm75 &= ~ (1 << (i - 0x48));
      LM_Sensor_Config.selected.lm75 &= ~ (1 << (i - 0x48));
    }
    else { /* Test for existance first */
      SMBus_Write(i,1 | (00 << 8),SMBUS_WORD_DATA); /* Set Config reg to 0x00 */
      LM75_Set(LM_PROBE_LM75_OS,i-0x48,LM75_TEMP_INIT_OS);
      LM75_Set(LM_PROBE_LM75_HYST,i-0x48,LM75_TEMP_INIT_HYST);
    }
  }
  return 0;
}

void LM75_Cleanup () {
#ifndef SMBUS_MODULE
   SMBus_Cleanup();
#endif
}

void LM75_Set(int what, int nr, long value) {

      if ( (nr < 0) || (nr > 7))  /* nr out of range */
        return;
      value = (value + 2) / 5 /* Now in half-degrees */;
      if (value < 0)
        value+=0x200; 
      value = ((value & 0x1fe)>>1)+ ((value & 1)<<15); /* Now in SMBus format */
      if (what == LM_PROBE_LM75_HYST) { /* Set Thyst=value */
        SMBus_Write(nr + 0x48,2 | (value << 8), SMBUS_WORD_DATA); 
        LM_Sensor_Data.lm75[nr].temphyst = value;
      }
      else if (what == LM_PROBE_LM75_OS) { /* Set Tos=value */
        SMBus_Write(nr + 0x48,3 | (value << 8), SMBUS_WORD_DATA); 
        LM_Sensor_Data.lm75[nr].tempos = value;
      }
}

long LM75_Get(int what, int nr) {
    if ( (nr < 0) || (nr > 7))  /* nr out of range */
      return 0;
    if (what == LM_PROBE_LM75_VALUE)
      return calc_temp(LM_Sensor_Data.lm75[nr].temp);
    else if (what == LM_PROBE_LM75_HYST)
      return calc_temp(LM_Sensor_Data.lm75[nr].temphyst);
    else if (what == LM_PROBE_LM75_OS)
      return calc_temp(LM_Sensor_Data.lm75[nr].tempos);
    else
      return 0; /* what out of range */
  
}

/* Input: a LM75 register value.
   Output: the temperature in tenth of degrees */
int calc_temp(int value) {
   value=value >> 7;
   if (value > 0x0191)
      value= - ( 0x0200 - value );
   return value*5;
}

int LM75_Print_Values (char *buf) {
  int i,value,value2,value3;
  int len=0;

  if (LM_Sensor_Data.valid) {

    for (i=0; i < MAX_LM75 ; i++) 
      if (LM_Sensor_Config.selected.lm75 & (1 << i)) {
        value=LM75_Get(LM_PROBE_LM75_VALUE,i);
        value2=LM75_Get(LM_PROBE_LM75_OS,i);
        value3=LM75_Get(LM_PROBE_LM75_HYST,i);
        len += sprintf(buf+len, 
                      "%s reports %d.%d degrees C (Tos=%d.%d, Thyst=%d.%d)\n",
                      LM_Sensor_Config.lm75_label[i],
                      value / 10, abs(value) % 10,
                      value2 / 10, abs(value2) % 10, 
                      value3 / 10, abs(value3) % 10);
#ifdef DEBUG
        len += sprintf(buf+len, "%s reg 1 (configuration register) reports 0x%X\n",
               LM_Sensor_Config.lm75_label[i],LM_Sensor_Data.lm75[i].config);
#endif
      }
  }
  return len;
}

void LM75_Update_Values (void) {

  int i;
  int value,smbus_addr;

/* Some very ugly code: if we have probed a LM75 location before, and 
   nothing was found, we do not probe it again. */
  for (i = 0 ; i < 8 ; i ++)
    if (LM_Sensor_Present.lm75 & (1 << i)) {
      smbus_addr=i+0x048; 
      /* Clear configuration reg */
      SMBus_Write(smbus_addr,0x000001,SMBUS_BYTE_DATA);
      value=SMBus_Read(smbus_addr,0,SMBUS_WORD_DATA);
      if (value != -1) {
        LM_Sensor_Data.lm75[i].temp=value;
#ifdef DEBUG
        LM_Sensor_Data.lm75[i].config=SMBus_Read(smbus_addr,1,SMBUS_WORD_DATA);
#endif
        LM_Sensor_Data.lm75[i].temphyst=SMBus_Read(smbus_addr,2,SMBUS_WORD_DATA);
        LM_Sensor_Data.lm75[i].tempos=SMBus_Read(smbus_addr,3,SMBUS_WORD_DATA);
      } else {
        LM_Sensor_Present.lm75 &= ~ (1 << i);
        LM_Sensor_Config.selected.lm75 &= ~ (1 << i);
      }
    }
}
